package core_unix

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Epoll.FlagsSource

Sourcetype t

An Epoll.Flags.t is an immutable set of flags for which one can register interest in a file descriptor. It is implemented as a bitmask, and so all operations (+, -, etc.) are constant time with no allocation.

sexp_of_t produces a human-readable list of bits, e.g., "(in out)".

include Flags.S with type t := t
include Sexplib0.Sexpable.S with type t := t
Sourceval t_of_sexp : Sexplib0.Sexp.t -> t
Sourceval sexp_of_t : t -> Sexplib0.Sexp.t

consistent with subset

include Core.Comparable.S with type t := t
include Base.Comparable.S with type t := t
include Base.Comparisons.S with type t := t
include Base.Comparisons.Infix with type t := t
Sourceval (>=) : t -> t -> bool
Sourceval (<=) : t -> t -> bool
Sourceval (=) : t -> t -> bool
Sourceval (>) : t -> t -> bool
Sourceval (<) : t -> t -> bool
Sourceval (<>) : t -> t -> bool
Sourceval equal : t -> t -> bool
Sourceval compare : t -> t -> int

compare t1 t2 returns 0 if t1 is equal to t2, a negative integer if t1 is less than t2, and a positive integer if t1 is greater than t2.

Sourceval min : t -> t -> t
Sourceval max : t -> t -> t
Sourceval ascending : t -> t -> int

ascending is identical to compare. descending x y = ascending y x. These are intended to be mnemonic when used like List.sort ~compare:ascending and List.sort ~cmp:descending, since they cause the list to be sorted in ascending or descending order, respectively.

Sourceval descending : t -> t -> int
Sourceval between : t -> low:t -> high:t -> bool

between t ~low ~high means low <= t <= high

Sourceval clamp_exn : t -> min:t -> max:t -> t

clamp_exn t ~min ~max returns t', the closest value to t such that between t' ~low:min ~high:max is true.

Raises if not (min <= max).

Sourceval clamp : t -> min:t -> max:t -> t Base.Or_error.t
include Base.Comparator.S with type t := t
Sourcetype comparator_witness
Sourceval validate_lbound : min:t Core.Maybe_bound.t -> t Validate.check
Sourceval validate_ubound : max:t Core.Maybe_bound.t -> t Validate.check
Sourceval validate_bound : min:t Core.Maybe_bound.t -> max:t Core.Maybe_bound.t -> t Validate.check
Sourceval to_flag_list : t -> t * string list
Sourceval of_int : int -> t
Sourceval to_int_exn : t -> int
Sourceval empty : t
Sourceval (+) : t -> t -> t

set union, bitwise or

Sourceval (-) : t -> t -> t

set difference. Although we use operators + and -, they do not satisfy the usual arithmetic equations, e.g. x - y = x + (empty - y) does not hold.

Sourceval intersect : t -> t -> t

bitwise and

Sourceval complement : t -> t

bitwise not

Sourceval is_empty : t -> bool
Sourceval do_intersect : t -> t -> bool
Sourceval are_disjoint : t -> t -> bool
Sourceval is_subset : t -> of_:t -> bool

is_subset t ~of_ is t = intersect t of_

Sourcemodule Unstable : sig ... end

The names of the flags match the man pages. E.g. in_ = "EPOLLIN", out = "EPOLLOUT", etc.

Sourceval none : t

Associated fd is readable

Sourceval in_ : t

Associated fd is readable

Sourceval out : t

Associated fd is writable

Sourceval pri : t

Urgent data available

Sourceval err : t

Error condition (always on, no need to set it)

Sourceval hup : t

Hang up happened (always on)

Sourceval et : t

Edge-Triggered behavior (see man page)

Sourceval oneshot : t

One-shot behavior for the associated fd