package core_unix

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Command_unix.PathSource

Sourcetype t

Path.t is a top-level executable name and sequence of subcommand names that can be used to identify a command.

Sourceval create : path_to_exe:string -> t

create creates a path from a toplevel executable given by path_to_exe.

Sourceval append : t -> subcommand:string -> t

append appends a subcommand to t.

Sourceval parts : t -> string list

parts returns a list containing the path's executable name followed by its subcommands.